==> runtime-simulator-0.7.0.tgz: package/src/polyfills/fetch.ts <==
/**
 * @license
 * Copyright 2022 Dynatrace LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import { readFileSync } from 'fs';
import { join } from 'path';
import type { RequestInfo, RequestInit } from 'node-fetch';
import fetch, { Request, Response } from 'node-fetch';
import { ContextConfiguration } from '../context';

/** The Fetch API provides an interface for fetching resources */
export default function (
  cwd: string,
  environmentUrl: string,
  config?: ContextConfiguration,
) {
  return (
    requestInfo: RequestInfo,
    init: RequestInit = {},
  ): Promise<Response> => {
    const headers = init.headers || {};
    const token = config?.token ?? undefined;
    const appId = config?.appId ?? undefined;

    if (token && typeof requestInfo === 'string') {
      // If the request URL is an absolute pathname, we append the environmentUrl to
      // and add an authorization header containing the token to the request.
      // We add the Dt-App-Context header to tell the api gateway that an app is executing the REST request
      if (requestInfo.startsWith('/')) {
        headers['authorization'] = `Bearer ${token}`;
        headers['Dt-App-Context'] = 'local-dev-mode';
        headers['Dt-Dev-App-Id'] = appId;
        headers['Dt-Dev-Context'] = appId;
        const url = new URL(environmentUrl);

        return fetch(`${url.origin}${requestInfo}`, { ...init, headers });
      }
    }

    const request = new Request(requestInfo);
    const url = new URL(request.url);

    if (url.protocol === 'file:') {
      if (!process.env.DTP_TMP_FUNCTION_ASSET_DIR) {
        throw Error(
          'Something went wrong when trying to access a function asset',
        );
      }
      const file = readFileSync(
        join(process.env.DTP_TMP_FUNCTION_ASSET_DIR!, url.pathname),
      );

      return Promise.resolve(new Response(file, { statusText: 'OK' }));
    }

    return fetch(requestInfo, init);
  };
}

==> runtime-simulator-0.8.0.tgz: package/src/polyfills/fetch.ts <==
/**
 * @license
 * Copyright 2022 Dynatrace LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import { readFileSync } from 'fs';
import { join } from 'path';
import type { RequestInfo, RequestInit } from 'undici';
import { fetch, Request, Response, Headers } from 'undici';
import { ContextConfiguration } from '../context';

const FORBIDDEN_HEADERS = ['dt-dev-context', 'dt-app-context', 'dt-dev-app-id'];

/** The Fetch API provides an interface for fetching resources */
export default function (
  cwd: string,
  environmentUrl: string,
  config?: ContextConfiguration,
) {
  return (
    requestInfo: RequestInfo,
    init: RequestInit = {},
  ): Promise<Response> => {
    const headers = new Headers(init.headers ?? {});
    const token = config?.token;
    const appId = config?.appId;

    if (token && typeof requestInfo === 'string') {
      // If the request URL is an absolute pathname, we append the environmentUrl to
      // and add an authorization header containing the token to the request.
      // We add the Dt-App-Context header to tell the api gateway that an app is executing the REST request
      if (requestInfo.startsWith('/')) {
        headers.set('authorization', `Bearer ${token}`);
        const headerKeys = [...headers.keys()].map((key) => key.toLowerCase());
        FORBIDDEN_HEADERS.forEach((forbidden_header) => {
          if (headerKeys.includes(forbidden_header)) {
            throw new Error(
              `Forbidden request: Header '${forbidden_header}' not allowed in request`,
            );
          }
        });
        headers.set('Dt-App-Context', 'local-dev-mode');
        headers.set('Dt-Dev-App-Id', appId || '');
        headers.set('Dt-Dev-Context', appId || '');

        const url = new URL(environmentUrl);

        return fetch(`${url.origin}${requestInfo}`, { ...init, headers });
      }
    }

    const request = new Request(requestInfo);
    const url = new URL(request.url);

    if (url.protocol === 'file:') {
      if (!process.env.DTP_TMP_FUNCTION_ASSET_DIR) {
        throw Error(
          'Something went wrong when trying to access a function asset',
        );
      }
      const file = readFileSync(
        join(process.env.DTP_TMP_FUNCTION_ASSET_DIR!, url.pathname),
      );

      return Promise.resolve(new Response(file, { statusText: 'OK' }));
    }

    return fetch(requestInfo, init);
  };
}
